@@ -10,7 +10,7 @@ import requests
 from bs4 import BeautifulSoup
 import warnings
 import urllib
-
+from .utils import EleanorWarning
 from .mast import tic_by_contamination
 
 
@@ -26,6 +26,7 @@ def check_pointing(sector, camera, chip, path=None):
     searches = [
         's{0:04d}-{1}-{2}_tess_v2_pm.txt'.format(sector, camera, chip),
         'pointingModel_{0:04d}_{1}-{2}.txt'.format(sector, camera, chip),
+        'hlsp_eleanor_tess_ffi_postcard-s{0:04d}-{1}-{2}_tess_v2_pm.txt'.format(sector, camera, chip)
     ]
 
     # Checks a directory of pointing models, if it exists
@@ -37,7 +38,7 @@ def check_pointing(sector, camera, chip, path=None):
         pm = [i for i in pm_downloaded if search in i]
         if len(pm) > 0:
             return Table.read(os.path.join(pm_dir, pm[0]), format="ascii.basic")
-    warnings.warn("couldn't find pointing model")
+    warnings.warn("couldn't find pointing model", category=EleanorWarning)
 
 
 def load_pointing_model(pm_dir, sector, camera, chip):
@@ -68,7 +69,7 @@ def use_pointing_model(coords, pointing_model):
     return fhat
 
 
-def pm_quality(time, sector, camera, chip, pm=None):
+def pm_quality(time, sector, camera, chip, pm=None, pm_dir=None):
         """ Fits a line to the centroid motions using the pointing model.
             A quality flag is set if the centroid is > 2*sigma away from
                 the majority of the centroids.
@@ -119,7 +120,7 @@ def pm_quality(time, sector, camera, chip, pm=None):
 
 
 def set_quality_flags(ffi_start, ffi_stop, shortCad_fn, sector, camera, chip,
-                      pm=None):
+                      pm=None, pm_dir=None):
     """ Uses the quality flags in a 2-minute target to create quality flags
         in the postcards.
     We create our own quality flag as well, using our pointing model.
@@ -150,7 +151,8 @@ def set_quality_flags(ffi_start, ffi_stop, shortCad_fn, sector, camera, chip,
     convolve_ffi = np.array(convolve_ffi)
 
     flags    = np.bitwise_and(convolve_ffi, ffi_apply)
-    pm_flags = pm_quality(ffi_stop, sector, camera, chip, pm=pm) * 131072
+    pm_flags = pm_quality(ffi_stop, sector, camera,
+                          chip, pm=pm, pm_dir=pm_dir) * 131072
 
     pm_flags[ ((ffi_stop>1420.) & (ffi_stop < 1424.)) ] = 131072
 
