@@ -25,15 +25,18 @@ from .update import *
 __all__ = ['Source', 'multi_sectors']
 
 
-def multi_sectors(sectors, tic=None, gaia=None, coords=None, name=None, tc=False, local=False, post_dir=None, pm_dir=None,
+def multi_sectors(sectors, tic=None, gaia=None,
+                  coords=None, name=None, tc=False, local=False,
+                  post_dir=None, pm_dir=None,
                   metadata_path=None, tesscut_size=31):
-    """Obtain a list of Source objects for a single target, for each of multiple sectors for which the target was observed.
+    """Obtain a list of Source objects for a single target, for each of
+       multiple sectors for which the target was observed.
 
     Parameters
     ----------
     sectors : list or str
-        The list of sectors for which data should be returned, or `'all'` to return all sectors
-        for which there are data.
+        The list of sectors for which data should be returned, or `'all'` to
+        return all sectors for which there are data.
     tic : int, optional
         The TIC ID of the source.
     gaia : int, optional
@@ -41,8 +44,8 @@ def multi_sectors(sectors, tic=None, gaia=None, coords=None, name=None, tc=False
     coords : tuple, optional
         The (RA, Dec) coords of the object in degrees.
     tc : bool, optional
-        If True, use a TessCut cutout to produce postcards rather than downloading the eleanor
-        postcard data products.
+        If True, use a TessCut cutout to produce postcards rather than
+        downloading the eleanor postcard data products.
     tesscut_size : int, array-like, astropy.units.Quantity
         The size of the cutout array, when tc is True. Details can be seen in
         astroquery.mast.TesscutClass.download_cutouts
@@ -72,7 +75,8 @@ def multi_sectors(sectors, tic=None, gaia=None, coords=None, name=None, tc=False
 
     if (type(sectors) == list) or (type(sectors) == np.ndarray):
         for s in sectors:
-            star = Source(tic=tic, gaia=gaia, coords=coords, sector=int(s), tc=tc, local=local, post_dir=post_dir, pm_dir=pm_dir,
+            star = Source(tic=tic, gaia=gaia, coords=coords, sector=int(s), tc=tc,
+                          local=local, post_dir=post_dir, pm_dir=pm_dir,
                           metadata_path=metadata_path, tesscut_size=tesscut_size)
             if star.sector is not None:
                 objs.append(star)
@@ -371,7 +375,11 @@ class Source(object):
 
             if len(postcard_obs) > 0:
                 product_list = Observations.get_product_list(postcard_obs)
-                self.pointing = check_pointing(self.sector, self.camera, self.chip, self.pm_dir)
+                # we're just checking if the pointing file already exists. we don't need
+                # to be warned that it doesn't.
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore", category=EleanorWarning)
+                    self.pointing = check_pointing(self.sector, self.camera, self.chip, self.pm_dir)
 
                 if self.pointing is None:
                     extension = ["pc.fits", "bkg.fits", "pm.txt"]
@@ -436,7 +444,7 @@ class Source(object):
         fn_exists = self.search_tesscut(download_dir, coords)
 
         if fn_exists is None:
-            manifest = Tesscut.download_cutouts(coords, self.tesscut_size, sector=self.sector, path=download_dir)
+            manifest = Tesscut.download_cutouts(coordinates=coords, size=self.tesscut_size, sector=self.sector, path=download_dir)
             cutout = fits.open(manifest['Local Path'][0])
             self.postcard_path = manifest['Local Path'][0]
         else:
