@@ -15,14 +15,20 @@ from time import strftime
 from astropy.io import fits
 from scipy.stats import mode
 from scipy.signal import savgol_filter
+from scipy.interpolate import griddata
 from urllib.request import urlopen
 import os, sys, copy
 import os.path
 import warnings
 import pickle
 
+import tensorflow as tf
+from tqdm import tqdm
+
 from .ffi import use_pointing_model, load_pointing_model, centroid_quadratic
 from .postcard import Postcard, Postcard_tesscut
+from .models import Gaussian, Moffat
+from .utils import *
 
 __all__  = ['TargetData']
 
@@ -227,7 +233,10 @@ class TargetData(object):
                 self.get_cbvs()
 
                 self.create_apertures(self.tpf.shape[1], self.tpf.shape[2])
-
+                if self.all_apertures.size == 0:
+                    raise EdgeProblem('This star falls off the edge of the '
+                                      'detector and we cannot produce a light '
+                                      'curve.')
                 self.get_lightcurve()
 
                 if do_pca == True:
@@ -247,7 +256,8 @@ class TargetData(object):
 
 
     def get_time(self, coords):
-        """Gets time, including light travel time correction to solar system barycenter for object given location"""
+        """Gets time, including light travel time correction to solar system
+        barycenter for object given location"""
         t0 = self.post_obj.time - self.post_obj.barycorr
 
         ra = Angle(coords[0], u.deg)
@@ -313,7 +323,8 @@ class TargetData(object):
         x_upp_bkg = med_x+x_bkg_len + 1
 
         if height % 2 == 0 or width % 2 == 0:
-            warnings.warn('We force our TPFs to have an odd height and width so we can properly center our apertures.')
+            warnings.warn('We force our TPFs to have an odd height and width \
+            so we can properly center our apertures.')
 
         post_y_upp, post_x_upp = self.post_obj.dimensions[1], self.post_obj.dimensions[2]
 
@@ -354,7 +365,8 @@ class TargetData(object):
 
         if source.tc == False:
             if (x_low_lim==0) or (y_low_lim==0) or (x_upp_lim==post_x_upp) or (y_upp_lim==post_y_upp):
-                warnings.warn("The size postage stamp you are requesting falls off the edge of the postcard.")
+                warnings.warn("The size postage stamp you are requesting falls off \
+                the edge of the postcard.")
                 warnings.warn("WARNING: Your postage stamp may not be centered.")
 
             self.tpf     = post_flux[:, y_low_lim:y_upp_lim, x_low_lim:x_upp_lim]
@@ -373,6 +385,36 @@ class TargetData(object):
             self.tpf_err = post_err[: , y_low_lim:y_upp_lim, x_low_lim:x_upp_lim]
             self.tpf_err[np.isnan(self.tpf_err)] = np.inf
 
+            # there are NaNs (likely from saturated columns) in the postcard
+            # background measure
+            if not np.isfinite(self.bkg_tpf).all():
+                # go through every cadence and fill them in
+                tsteps = np.arange(self.bkg_tpf.shape[0])
+                for itime in tsteps:
+                    arr = self.bkg_tpf[itime, :, :]
+                    good = np.isfinite(self.bkg_tpf[itime, :, :])
+                    # this cadence is clean
+                    if good.all():
+                        continue
+                    xs, ys = np.meshgrid(np.arange(arr.shape[0]),
+                                         np.arange(arr.shape[1]), indexing='ij')
+                    # interpolate from nearby pixels
+                    fit = griddata((xs[good].flatten(), ys[good].flatten()),
+                                   arr[good].flatten(),
+                                   (xs[~good].flatten(), ys[~good].flatten()))
+                    # fill in with approximations
+                    self.bkg_tpf[itime, ~good] = fit
+                    # try again
+                    good = np.isfinite(self.bkg_tpf[itime, :, :])
+                    # must have NaNs along the edge, so linear interpolation fails
+                    if not good.all():
+                        # just fill in the edges with the same background
+                        # as the nearest good pixel
+                        fit = griddata((xs[good].flatten(), ys[good].flatten()),
+                                       arr[good].flatten(),
+                                       (xs[~good].flatten(),
+                                        ys[~good].flatten()), method='nearest')
+                        self.bkg_tpf[itime, ~good] = fit
 
         else:
             post_x_length, post_y_length = post_flux.shape[2], post_flux.shape[1]
@@ -467,11 +509,13 @@ class TargetData(object):
             lmask, lname = lap.to_mask(method='center').to_image(shape=shape), 'rectangle_{}'.format(int(deg))
             tmask, tname = tap.to_mask(method='center').to_image(shape=shape), 'L_{}'.format(int(deg))
 
-            all_apertures.append(lmask)
-            aperture_names.append(lname)
+            if lmask is not None and lmask.sum() > 0:
+                all_apertures.append(lmask)
+                aperture_names.append(lname)
 
-            all_apertures.append(tmask)
-            aperture_names.append(tname)
+            if tmask is not None and tmask.sum() > 0:
+                all_apertures.append(tmask)
+                aperture_names.append(tname)
 
             cap = circle_aperture(center, clist[i])
             rap = square_aperture(center, rlist[i], rlist[i], theta[i])
@@ -480,11 +524,13 @@ class TargetData(object):
                 cmask, cname = cap.to_mask(method=method).to_image(shape=shape), '{}_circle_{}'.format(clist[i], method)
                 rmask, rname = rap.to_mask(method=method).to_image(shape=shape), '{}_square_{}'.format(rlist[i], method)
 
-                all_apertures.append(cmask)
-                aperture_names.append(cname)
+                if cmask is not None and cmask.sum() > 0:
+                    all_apertures.append(cmask)
+                    aperture_names.append(cname)
 
-                all_apertures.append(rmask)
-                aperture_names.append(rname)
+                if rmask is not None and rmask.sum() > 0:
+                    all_apertures.append(rmask)
+                    aperture_names.append(rname)
 
             deg += 90
 
@@ -506,7 +552,8 @@ class TargetData(object):
         self.aperture_names = np.array(aperture_names)
 
         if height < default or width < default:
-            warnings.warn('WARNING: Making a TPF smaller than (13,13) may provide inadequate results.')
+            warnings.warn('WARNING: Making a TPF smaller than (13,13) may \
+                           provide inadequate results.')
 
 
     def bkg_subtraction(self, scope="tpf", sigma=2.5):
@@ -515,10 +562,13 @@ class TargetData(object):
         Parameters
         ----------
         scope : string, "tpf" or "postcard"
-            If `tpf`, will use data from the target pixel file only to estimate and remove the background.
-            If `postcard`, will use data from the entire postcard region to estimate and remove the background.
+            If `tpf`, will use data from the target pixel file only to estimate
+            and remove the background.
+            If `postcard`, will use data from the entire postcard region to
+            estimate and remove the background.
         sigma : float
-            The standard deviation cut used to determine which pixels are representative of the background in each cadence.
+            The standard deviation cut used to determine which pixels are
+            representative of the background in each cadence.
         """
         time = self.time
 
@@ -539,16 +589,20 @@ class TargetData(object):
 
     def get_lightcurve(self, aperture=None):
         """Extracts a light curve using the given aperture and TPF.
-        Can pass a user-defined aperture mask, otherwise determines which of a set of pre-determined apertures
+        Can pass a user-defined aperture mask, otherwise determines which of a
+        set of pre-determined apertures
         provides the lowest scatter in the light curve.
-        Produces a mask, a numpy.ndarray object of the same shape as the target pixel file, which every pixel assigned
+        Produces a mask, a numpy.ndarray object of the same shape as the target
+        pixel file, which every pixel assigned
         a weight in the range [0, 1].
 
         Parameters
         ----------
         aperture : numpy.ndarray
-            (`height`, `width`) array of floats in the range [0,1] with desired weights for each pixel to
-            create a light curve. If not set, ideal aperture is inferred automatically. If set, uses this
+            (`height`, `width`) array of floats in the range [0,1] with desired
+            weights for each pixel to
+            create a light curve. If not set, ideal aperture is inferred
+            automatically. If set, uses this
             aperture at the expense of all other set apertures.
         """
 
@@ -574,8 +628,10 @@ class TargetData(object):
         if self.aperture is not None:
             if np.shape(self.all_apertures[0]) != np.shape(self.aperture):
                 raise ValueError(
-                    "Passed aperture does not match the size of the TPF. Please correct and try again. "
-                    "Or, create a custom aperture using the function TargetData.custom_aperture(). See documentation for inputs.")
+                    "Passed aperture does not match the size of the TPF. Please \
+                    correct and try again. "
+                    "Or, create a custom aperture using the function \
+                    TargetData.custom_aperture(). See documentation for inputs.")
 
             self.all_apertures = np.zeros((1, np.shape(self.tpf[0])[0], np.shape(self.tpf[0])[1]))
             self.all_apertures[0] = self.aperture
@@ -623,13 +679,13 @@ class TargetData(object):
             ## Remove something from all_raw_lc before passing into jitter_corr ##
             try:
                 norm = np.nansum(self.all_apertures[a], axis=1)
-                all_corr_lc_pc_sub[a] = self.corrected_flux(flux=all_raw_lc_pc_sub[a]/np.nanmedian(all_raw_lc_pc_sub[a]),
+                all_corr_lc_pc_sub[a] = self.corrected_flux(flux=all_raw_lc_pc_sub[a]/np.nanmedian(np.abs(all_raw_lc_pc_sub[a])),
                                                            bkg=self.flux_bkg[:, None] * norm)
-                all_corr_lc_tpf_sub[a]= self.corrected_flux(flux=all_raw_lc_tpf_sub[a]/np.nanmedian(all_raw_lc_tpf_sub[a]),
+                all_corr_lc_tpf_sub[a]= self.corrected_flux(flux=all_raw_lc_tpf_sub[a]/np.nanmedian(np.abs(all_raw_lc_tpf_sub[a])),
                                                             bkg=self.tpf_flux_bkg[:, None] * norm)
 
                 if self.source_info.tc == False:
-                    all_corr_lc_tpf_2d_sub[a] = self.corrected_flux(flux=all_raw_lc_tpf_2d_sub[a]/np.nanmedian(all_raw_lc_tpf_2d_sub[a]),
+                    all_corr_lc_tpf_2d_sub[a] = self.corrected_flux(flux=all_raw_lc_tpf_2d_sub[a]/np.nanmedian(np.abs(all_raw_lc_tpf_2d_sub[a])),
                                                                     bkg=np.nansum(self.bkg_tpf*self.all_apertures[a], axis=(1,2)))
 
 
@@ -645,7 +701,6 @@ class TargetData(object):
                 stds_2d[a] = get_flattened_sigma(all_corr_lc_tpf_2d_sub[a][q][self.cal_cadences[0]:self.cal_cadences[1]])
                 all_corr_lc_tpf_2d_sub[a] = all_corr_lc_tpf_2d_sub[a] * np.nanmedian(all_raw_lc_tpf_2d_sub[a])
 
-
             all_corr_lc_pc_sub[a]  = all_corr_lc_pc_sub[a]  * np.nanmedian(all_raw_lc_pc_sub[a])
             all_corr_lc_tpf_sub[a] = all_corr_lc_tpf_sub[a] * np.nanmedian(all_raw_lc_tpf_sub[a])
 
@@ -673,8 +728,11 @@ class TargetData(object):
         best_ind_tpf = np.where(tpf_stds == np.nanmin(tpf_stds))[0][0]
         best_ind_pc  = np.where(pc_stds == np.nanmin(pc_stds))[0][0]
 
-        if self.source_info.tc == False:
-            best_ind_2d = np.where(stds_2d == np.nanmin(stds_2d))[0][0]
+        if not self.source_info.tc:
+            if np.isfinite(stds_2d).any():
+                best_ind_2d = np.where(stds_2d == np.nanmin(stds_2d))[0][0]
+            else:
+                best_ind_2d = None
         else:
             best_ind_2d = None
 
@@ -729,15 +787,19 @@ class TargetData(object):
 
 
     def get_cbvs(self):
-        """ Obtains the cotrending basis vectors (CBVs) as convolved down from the short-cadence targets.
+        """
+        Obtains the cotrending basis vectors (CBVs) as convolved down
+        from the short-cadence targets.
+
         Parameters
         ----------
         """
 
         try:
-            matrix_file = np.loadtxt(self.source_info.eleanorpath + '/metadata/s{0:04d}/cbv_components_s{0:04d}_{1:04d}_{2:04d}.txt'.format(self.source_info.sector,
-                                                                                                                                                         self.source_info.camera,
-                                                                                                                                                         self.source_info.chip))
+            matrix_file = np.loadtxt(self.source_info.eleanorpath +
+            '/metadata/s{0:04d}/cbv_components_s{0:04d}_{1:04d}_{2:04d}.txt'.format(self.source_info.sector,
+                                                                                    self.source_info.camera,
+                                                                                    self.source_info.chip))
             cbvs = np.asarray(matrix_file)
             self.cbvs = np.reshape(cbvs, (len(self.time), 16))
 
@@ -748,10 +810,12 @@ class TargetData(object):
 
     def center_of_mass(self):
         """
-        Calculates the position of the source across all cadences using `muchbettermoments` and `self.best_aperture`.
+        Calculates the position of the source across all cadences using
+        `muchbettermoments` and `self.best_aperture`.
 
         Finds the brightest pixel in a (`height`, `width`) region summed up over all cadence.
-        Searches a smaller (3x3) region around this pixel at each cadence and uses `muchbettermoments` to find the maximum.
+        Searches a smaller (3x3) region around this pixel at each cadence and
+        uses `muchbettermoments` to find the maximum.
         """
 
         self.x_com = []
@@ -783,7 +847,6 @@ class TargetData(object):
         return
 
 
-
     def set_quality(self):
         """ Reads in quality flags set in the postcard
         """
@@ -791,7 +854,7 @@ class TargetData(object):
 
         self.quality[np.nansum(self.tpf, axis=(1,2)) == 0] = 128
 
-        bkgvar = np.std(self.bkg_tpf, axis=(1,2))/(np.nansum(self.bkg_tpf, axis=(1,2)))
+        bkgvar = np.nanstd(self.bkg_tpf, axis=(1,2))/(np.nansum(self.bkg_tpf, axis=(1,2)))
         bkgmask = sigma_clip(bkgvar, masked=True, sigma=5.0)
         self.quality = np.bitwise_or(self.quality.astype(int), (bkgmask.mask * 2**18).astype(int))
 
@@ -840,11 +903,6 @@ class TargetData(object):
             target, effectively masking other nearby, bright stars. This strategy appears to do a
             reasonable job estimating the background more accurately in relatively crowded regions.
         """
-
-        import tensorflow as tf
-        from .models import Gaussian, Moffat
-        from tqdm import tqdm
-
         tf.logging.set_verbosity(tf.logging.ERROR)
 
         if data_arr is None:
@@ -880,7 +938,6 @@ class TargetData(object):
         if len(yc) != nstars:
             raise ValueError('yc must have length nstars')
 
-
         flux = tf.Variable(np.ones(nstars)*np.max(data_arr[0]), dtype=tf.float64)
         bkg = tf.Variable(bkg_arr[0], dtype=tf.float64)
         xshift = tf.Variable(0.0, dtype=tf.float64)
@@ -938,10 +995,8 @@ class TargetData(object):
                              beta: (0, 10)
                             }
 
-
             betaout = np.zeros(len(data_arr))
 
-
         else:
             raise ValueError('This model is not incorporated yet!') # we probably want this to be a warning actually,
                                                                     # and a gentle return
@@ -1001,7 +1056,6 @@ class TargetData(object):
                 llout[i] = sess.run(nll, feed_dict={data:data_arr[i], derr:err_arr[i], bkgval:bkg_arr[i]})
                 betaout[i] = sess.run(beta)
 
-
         sess.close()
 
         self.psf_flux = fout[:,0]
@@ -1031,8 +1085,6 @@ class TargetData(object):
                 self.all_psf = fout
         return
 
-
-
     def custom_aperture(self, shape=None, r=0.0, h=0.0, w=0.0, theta=0.0, pos=None, method='exact'):
         """
         Creates a custom circular or rectangular aperture of arbitrary size.
@@ -1128,7 +1180,8 @@ class TargetData(object):
         Parameters
         ----------
         skip: float, optional
-            The length of time in days at the start of each orbit to skip in determining optimal model weights.
+            The length of time in days at the start of each orbit to skip in
+            determining optimal model weights.
             Default is 0.62 days.
         modes: int, optional
             If doing a PCA-based correction, the number of cotrending basis vectors to regress against.
@@ -1136,8 +1189,10 @@ class TargetData(object):
         pca : bool, optional
             Allows users to decide whether or not to use PCA analysis. Default is False.
         regressors : numpy.ndarray or str
-            Extra data to regress against in the correction. Should be shape (len(data.time), N) or `'corner'`.
-            If `'corner'` will use the four corner pixels of the TPF as extra information in the regression.
+            Extra data to regress against in the correction. Should be shape
+            (len(data.time), N) or `'corner'`.
+            If `'corner'` will use the four corner pixels of the TPF as extra
+            information in the regression.
         """
         self.modes = modes
 
@@ -1235,6 +1290,7 @@ class TargetData(object):
             x = xhat(cm, norm_l[skip:])
             fmod = fhat(x, cm_full)
             lc_pred = (fmod+1)
+
             return lc_pred*medval
 
 
@@ -1692,7 +1748,7 @@ def get_flattened_sigma(y, maxiter=100, window_size=51, nsigma=4):
     sig = np.std(y)
     m = np.ones_like(y, dtype=bool)
     n = len(y)
-    for n in range(maxiter):
+    for _ in range(maxiter):
         sig = np.std(y[m])
         m = np.abs(y) < nsigma * sig
         if m.sum() == n:
